Fio 3.15
[fio.git] / flist.h
diff --git a/flist.h b/flist.h
index f28a5ab7ef27250151cc1f1a5678b68fff519d5a..5437cd80da85ab7581b2282f52669510ef682d20 100644 (file)
--- a/flist.h
+++ b/flist.h
@@ -1,15 +1,11 @@
 #ifndef _LINUX_FLIST_H
 #define _LINUX_FLIST_H
 
-#undef offsetof
-#ifdef __compiler_offsetof
-#define offsetof(TYPE,MEMBER) __compiler_offsetof(TYPE,MEMBER)
-#else
-#define offsetof(TYPE, MEMBER) ((size_t) &((TYPE *)0)->MEMBER)
-#endif
+#include <stdlib.h>
+#include <stddef.h>
 
-#define container_of(ptr, type, member) ({                     \
-       const typeof( ((type *)0)->member ) *__mptr = (ptr);    \
+#define container_of(ptr, type, member)  ({                    \
+       const __typeof__( ((type *)0)->member ) *__mptr = (ptr);        \
        (type *)( (char *)__mptr - offsetof(type,member) );})
 
 /*
@@ -41,33 +37,34 @@ struct flist_head {
  * This is only for internal list manipulation where we know
  * the prev/next entries already!
  */
-static inline void __flist_add(struct flist_head *new,
+static inline void __flist_add(struct flist_head *new_entry,
                               struct flist_head *prev,
                               struct flist_head *next)
 {
-       next->prev = new;
-       new->next = next;
-       new->prev = prev;
-       prev->next = new;
+       next->prev = new_entry;
+       new_entry->next = next;
+       new_entry->prev = prev;
+       prev->next = new_entry;
 }
 
 /**
  * flist_add - add a new entry
* @new: new entry to be added
  * @head: list head to add it after
  *
  * Insert a new entry after the specified head.
  * This is good for implementing stacks.
  */
-static inline void flist_add(struct flist_head *new, struct flist_head *head)
+static inline void flist_add(struct flist_head *new_entry,
+                             struct flist_head *head)
 {
-       __flist_add(new, head, head->next);
+       __flist_add(new_entry, head, head->next);
 }
 
-static inline void flist_add_tail(struct flist_head *new,
+static inline void flist_add_tail(struct flist_head *new_entry,
                                  struct flist_head *head)
 {
-       __flist_add(new, head->prev, head);
+       __flist_add(new_entry, head->prev, head);
 }
 
 /*
@@ -116,6 +113,52 @@ static inline int flist_empty(const struct flist_head *head)
        return head->next == head;
 }
 
+static inline void __flist_splice(const struct flist_head *list,
+                                 struct flist_head *prev,
+                                 struct flist_head *next)
+{
+       struct flist_head *first = list->next;
+       struct flist_head *last = list->prev;
+
+       first->prev = prev;
+       prev->next = first;
+
+       last->next = next;
+       next->prev = last;
+}
+
+static inline void flist_splice(const struct flist_head *list,
+                               struct flist_head *head)
+{
+       if (!flist_empty(list))
+               __flist_splice(list, head, head->next);
+}
+
+static inline void flist_splice_tail(struct flist_head *list,
+                                    struct flist_head *head)
+{
+       if (!flist_empty(list))
+               __flist_splice(list, head->prev, head);
+}
+
+static inline void flist_splice_tail_init(struct flist_head *list,
+                                         struct flist_head *head)
+{
+       if (!flist_empty(list)) {
+               __flist_splice(list, head->prev, head);
+               INIT_FLIST_HEAD(list);
+       }
+}
+
+static inline void flist_splice_init(struct flist_head *list,
+                                   struct flist_head *head)
+{
+       if (!flist_empty(list)) {
+               __flist_splice(list, head, head->next);
+               INIT_FLIST_HEAD(list);
+       }
+}
+
 /**
  * flist_entry - get the struct for this entry
  * @ptr:       the &struct flist_head pointer.
@@ -125,6 +168,12 @@ static inline int flist_empty(const struct flist_head *head)
 #define flist_entry(ptr, type, member) \
        container_of(ptr, type, member)
 
+#define flist_first_entry(ptr, type, member) \
+       flist_entry((ptr)->next, type, member)
+
+#define flist_last_entry(ptr, type, member) \
+       flist_entry((ptr)->prev, type, member)
+
 /**
  * flist_for_each      -       iterate over a list
  * @pos:       the &struct flist_head to use as a loop counter.
@@ -143,4 +192,7 @@ static inline int flist_empty(const struct flist_head *head)
        for (pos = (head)->next, n = pos->next; pos != (head); \
                pos = n, n = pos->next)
 
+extern void flist_sort(void *priv, struct flist_head *head,
+       int (*cmp)(void *priv, struct flist_head *a, struct flist_head *b));
+
 #endif
@@ -41,33 +37,34 @@ struct flist_head {
 /**
  * flist_add - add a new entry
- * @new: new entry to be added
+ * @new_entry: new entry to be added
  * @head: list head to add it after
  *